#version 330 core

layout(location = 0) in vec3 position;
layout(location = 1) in vec2 texCoords;
layout(location = 2) in vec3 normal;
layout(location = 3) in vec3 tangent;
layout(location = 4) in vec3 bitangent;

out vec2 UV;															// out vec3 FragPos = position world_space
out vec3 Position_worldspace;
out vec3 Normal;
out vec3 EyeDirection_cameraspace;
out vec3 LightDirection_cameraspace;

out vec3 LightDirection_tangentspace;
out vec3 EyeDirection_tangentspace;

out vec4 FragPosLightSpace;

uniform mat4 MVP;
uniform mat4 V;
uniform mat4 M;
uniform mat3 MV3x3;
uniform vec3 LightPosition_worldspace;
uniform vec3 CameraPosition;
uniform mat4 lightSpaceMatrix;

void main() {
	gl_Position =  MVP * vec4(position,1);
	
	Position_worldspace = (M * vec4(position,1)).xyz;
	
	vec3 vertexPosition_cameraspace = ( V * M * vec4(position,1)).xyz;
	EyeDirection_cameraspace = CameraPosition - vertexPosition_cameraspace;

	vec3 LightPosition_cameraspace = ( V * vec4(LightPosition_worldspace,1)).xyz;
	LightDirection_cameraspace = LightPosition_cameraspace + EyeDirection_cameraspace;
	
	Normal = transpose(inverse(mat3(M))) * normal;
	UV = texCoords;
	
	vec3 vertexTangent_cameraspace = MV3x3 * tangent;
	vec3 vertexBitangent_cameraspace = MV3x3 * bitangent;
	vec3 vertexNormal_cameraspace = MV3x3 * normal;
	
	mat3 TBN = transpose(mat3(
		vertexTangent_cameraspace,
		vertexBitangent_cameraspace,
		vertexNormal_cameraspace	
	));

	LightDirection_tangentspace = TBN * LightDirection_cameraspace;
	EyeDirection_tangentspace =  TBN * EyeDirection_cameraspace;
	
	FragPosLightSpace = lightSpaceMatrix * vec4(Position_worldspace, 1.0);
}

